# -*- coding: utf-8 -*-
#
# Configuration file for the Sphinx documentation builder.
#
# This file does only contain a selection of the most common options. For a
# full list see the documentation:
# http://www.sphinx-doc.org/en/master/config

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys
sys.path.insert(0, os.path.abspath(".."))
sys.path.insert(0, os.path.abspath("_themes"))

import pinn

# -- Project information -----------------------------------------------------

project = u'pinn'
copyright = u'2019, Pinn Technologies, Inc'
author = u'Pinn Technologies, Inc'

# The short X.Y version
version = pinn.__version__
# The full version, including alpha/beta/rc tags
release = pinn.__version__


# -- General configuration ---------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.viewcode',
    'sphinx.ext.autodoc',
    'sphinx.ext.coverage',
    'sphinx.ext.napoleon'
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = None

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = [u'_build', 'Thumbs.db', '.DS_Store']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'flasky'


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
html_theme_options = {
    'logo_only': True,
    'style_nav_header_background': '#FFFFFF',
    'display_version': False,
}

html_logo = './_static/pinn-python-logo-small.png'
html_favicon = './_static/favicon.ico'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# Custom sidebar templates, must be a dictionary that maps document names
# to template names.
#
# The default sidebars (for documents that don't match any pattern) are
# defined by theme itself.  Builtin themes are using these templates by
# default: ``['localtoc.html', 'relations.html', 'sourcelink.html',
# 'searchbox.html']``.
#
# html_sidebars = {}


# -- Options for HTMLHelp output ---------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = 'pinndoc'


# -- Options for LaTeX output ------------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',

    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, 'pinn.tex', u'pinn Documentation',
     u'Pinn Technologies, Inc', 'manual'),
]


# -- Options for manual page output ------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'pinn', u'pinn Documentation',
     [author], 1)
]


# -- Options for Texinfo output ----------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'pinn', u'pinn Documentation',
     author, 'pinn', 'One line description of project.',
     'Miscellaneous'),
]


# -- Options for Epub output -------------------------------------------------

# Bibliographic Dublin Core info.
epub_title = project

# The unique identifier of the text. This can be a ISBN number
# or the project homepage.
#
# epub_identifier = ''

# A unique identification for the text.
#
# epub_uid = ''

# A list of files that should not be packed into the epub file.
epub_exclude_files = ['search.html']


# -- Extension configuration -------------------------------------------------



"""
:copyright: (c) 2019 Pinn Technologies, Inc.
:license: MIT
"""

import jwt
from .errors import IDTokenVerificationError


class IDToken(object):
    """Provides functionality for verifying an incoming Pinn ID token."""

    @staticmethod
    def verify(id_token, amr):
        """Verify a new incoming Pinn ID token value.

        Args:
            id_token (str): The ID Token received from a Pinn SDK
            amr (list): List of required authentication methods required

        Returns:
            dict: A dictionary of the verified claims
        """
        from . import secret_key, api_host
        if secret_key is None:
            raise ValueError('Pinn `secret_key` has not been set')
        try:
            claims = jwt.decode(id_token, key=secret_key)
        except Exception as e:
            raise IDTokenVerificationError(str(e))
        if 'sub' not in claims:
            raise IDTokenVerificationError('`sub` claim not provided in JWT')
        if 'amr' not in claims:
            raise IDTokenVerificationError('`amr` claim not provided in JWT')
        if 'iss' not in claims:
            raise IDTokenVerificationError('`iss` claim not provided in JWT')
        if claims['iss'] != api_host:
            raise IDTokenVerificationError(
                '`iss` claim provided was {} and was expected to be {}'.format(claims['iss'],
                                                                               api_host))
        if not set(amr).issubset(set(claims['amr'])):
            raise IDTokenVerificationError(
                '`amr` was invalid and did not contain all methods required.' +
                'Requested: {}, Received: {}'.format(amr, claims['amr']))
        return claims


class WebhookSignature(object):
    pass



"""
:copyright: (c) 2019 Pinn Technologies, Inc.
:license: MIT
"""

from ..requester import Requester
from .list import List


class WebhookEndpoint(object):
    """Webhook endpoint resource and interface.

    Attributes:
        response (dict): Underlying dictionary response
        object (str): Identifier for the resource
        webhook_endpoint_id (str): Unique ID for the webhook endpoint
        created_at (int): Unix timestamp in seconds for when the key was created
        created_at (int): Unix timestamp in seconds for when the key was updated
        status (str): The status of the webhook endpoint
        url (str): Fully qualified endpoint URL where webhook events are delivered to
        events (list): The list of events this webhook is subscribed to
        livemode (bool): True if webhook endpoint is in live environment
        secret (str): A secret to authenticate the enroll request from Pinn mobile SDK
    """

    OBJECT_NAME = 'webhook_endpoint'
    endpoint = '/v1/webhook_endpoints'

    def __init__(self, response):
        """Initialize a user model with an API response."""
        self.response = response
        self.object = response['object']
        self.webhook_endpoint_id = response['webhook_endpoint_id']
        self.created_at = response['created_at']
        self.updated_at = response['updated_at']
        self.status = response['status']
        self.url = response['url']
        self.events = response['events']
        self.livemode = response['livemode']
        self.secret = response['secret']

    @classmethod
    def create(cls, url, events):
        """Create Pinn webhook endpoint."""
        data = {'url': url,
                'events': events}
        return WebhookEndpoint(Requester.post(cls.endpoint, data=data))

    @classmethod
    def update(cls, webhook_endpoint_id, url=None, events=None, status=None):
        """Update Pinn webhook endpoint."""
        data = {}
        if url:
            data['url'] = url
        if events:
            data['events'] = events
        if status:
            data['status'] = status
        return WebhookEndpoint(Requester.patch(cls.endpoint + '/' + webhook_endpoint_id, data=data))

    @classmethod
    def list(cls, limit=None, starting_after=None):
        """List created Pinn logs."""
        response = Requester.get(cls.endpoint, params={'limit': limit,
                                                       'starting_after': starting_after})
        return List(response, WebhookEndpoint, limit)

    @classmethod
    def retrieve(cls, webhook_endpoint_id):
        """Retrieve an webhook_endpoint with a provided webhook_endpoint ID."""
        return WebhookEndpoint(Requester.get(cls.endpoint + '/' + webhook_endpoint_id))

    @classmethod
    def delete(cls, webhook_endpoint_id):
        """Delete a webhook_endpoint with a provided webhook_endpoint ID."""
        return Requester.delete(cls.endpoint + '/' + webhook_endpoint_id)



"""
:copyright: (c) 2019 Pinn Technologies, Inc.
:license: MIT
"""

from .list import List
from .log import Log
from ..requester import Requester


class Device(object):
    """Device resource and interface.

    Attributes:
        response (dict): Underlying dictionary response
        object (str): Identifier for the resource
        device_id (str): Unique ID for the device
        user_id (str): ID of the user the device is registered to
        created_at (int): Unix timestamp in seconds for when the app was created
        updated_at (int): Unix timestamp in seconds for when the app was last updated
        name (str): Human readable name of the device
        make (str): Manufacturer make of the device
        model (str): Model identifier of the device
        platform (str): Either `ios` or `android`
        platform_version (str): Last recorded platform version of the device
        framework_version (str): Last recorded framework version for the device
        push_notification_token (str, optional): Device APNS or FCM push notification token
    """

    OBJECT_NAME = 'key'
    endpoint = '/v1/devices'

    def __init__(self, response):
        """Initialize a user model with an API response."""
        self.response = response
        self.object = response['object']
        self.device_id = response['device_id']
        self.user_id = response['user_id']
        self.created_at = response['created_at']
        self.updated_at = response['updated_at']
        self.name = response['name']
        self.make = response['make']
        self.model = response['model']
        self.platform = response['platform']
        self.platform_version = response['platform_version']
        self.framework_version = response['framework_version']
        self.push_notification_token = response.get('push_notification_token')

    @classmethod
    def list(cls, limit=None, starting_after=None):
        """List created Pinn users."""
        response = Requester.get(cls.endpoint, params={'limit': limit,
                                                       'starting_after': starting_after})
        return List(response, Device, limit)

    @classmethod
    def list_logs(cls, device_id, limit=None, starting_after=None):
        """List created Pinn users."""
        endpoint = cls.endpoint + '/' + device_id + '/logs'
        response = Requester.get(endpoint, params={'limit': limit,
                                                   'starting_after': starting_after})
        return List(response, Log, limit)

    @classmethod
    def retrieve(cls, device_id):
        """Retrieve a specific key for a given user."""
        return Device(Requester.get(cls.endpoint + '/' + device_id))

    @classmethod
    def delete(cls, device_id):
        """Remove a key from the user's keychain."""
        return Requester.delete(cls.endpoint + '/' + device_id)



# :copyright: (c) 2019 Pinn Technologies, Inc.
# :license: MIT

import requests


class PinnError(Exception):
    """Base exception class for a Pinn API Error."""

    def __init__(self, response):
        """Create a PinnError object with a response dictionary object."""
        try:
            error = response.json()['error']
            self._message = error["message"]
            self.code = error["code"]
            self.type = error["type"]
        except:  # NOQA FIXME
            self._message = response.reason
            self.code = response.status_code
            self.type = PINN_ERROR_CODE_MAP[response.status_code]
        super(PinnError, self).__init__(self._message)

    @staticmethod
    def from_response(response):
        """Create an error of the right PinnError subclass from an API response."""
        if response.status_code == requests.codes.internal_server_error:
            return APIError(response)
        if response.status_code in [requests.codes.bad_gateway,
                                    requests.codes.service_unavailable,
                                    requests.codes.gateway_timeout]:
            return APIConnectionError(response)
        error = response.json()["error"]
        cls = PINN_ERROR_TYPE_MAP.get(error['type'], PinnError)
        return cls(response)


class InvalidRequestError(PinnError):
    """The request was malformed and could not be understood by the server.

    This could mean that an incorrect type was sent for a parameter, an invalid HTTP
    method was used or that the resource you are requesting does not exist. Do not
    attempt to retry the request.
    """
    pass


class AuthenticationError(PinnError):
    """The secret key set is invalid and cannot be used to authenticate your request."""
    pass


class RequestFailedError(PinnError):
    """The request was properly formed with valid syntax, but could not be performed.

    This occurs in scenarios where circumstances prevent an operation from being executed,
    typically this can be situations like invalid resource state to perform the op.
    """
    pass


class ForbiddenError(PinnError):
    """This error is returned if you attempt to access a resource you don't own.

    Do not attempt to retry this error, and if you receieve this error it likely
    means you have an incorrect `secret_key` set on the library.
    """
    pass


class RateLimitError(PinnError):
    """The API rate limit has been exceeded.

    You should attempt to retry this request after some time has passed.
    """
    pass


class APIError(PinnError):
    """An internal server error caused the request to fail.

    Pinn is automatically notified if issues are encountered that are internal to our
    service. You should retry this request in case the error was intermittent.
    """
    pass


class APIConnectionError(PinnError):
    """The Pinn host is unreachable, you can and should retry this request"""
    pass


PINN_ERROR_CODE_MAP = {
    requests.codes.internal_server_error: 'internal_server_error',  # 500
    requests.codes.service_unavailable: 'service_unavailable',      # 502
    requests.codes.gateway_timeout: 'gateway_timeout',              # 503
    requests.codes.bad_gateway: 'bad_gateway'                       # 504
}

PINN_ERROR_TYPE_MAP = {
    'bad_request': InvalidRequestError,          # 400
    'unauthorized': AuthenticationError,         # 401
    'forbidden': ForbiddenError,                 # 403
    'not_found': InvalidRequestError,            # 404
    'method_not_allowed': InvalidRequestError,   # 405
    'conflict': RequestFailedError,              # 409
    'unprocessable_entity': RequestFailedError,  # 422
    'rate_limit_exceeded': RateLimitError,       # 429
    'internal_server_error': APIError,           # 500
    'service_unavailable': APIConnectionError,   # 502
    'bad_gateway': APIConnectionError,           # 503
    'gateway_timeout': APIConnectionError        # 504
}


# Non API Errors
# --------------
class IDTokenVerificationError(Exception):
    """This exception can be raised during an attempt to verify an ID token.

    Potentially this can be from:
        - An invalid JWT signature
        - Missing required claims
        - `amr` claim mismatch from expectations
    """
    pass


class ConfigurationError(Exception):
    """This error is thrown fast if the library is misconfigured.

    For example, if you attempt to call the library without a `secret_key` value
    set a ConfigurationError will be thrown.
    """
    pass



# :copyright: (c) 2019 Pinn Technologies, Inc.
# :license: MIT

import os

# Package Information
# -------------------
__title__ = 'pinn'
__author__ = 'Pinn Technologies, Inc.'
__license__ = 'MIT'
__copyright__ = 'Copyright 2019 Pinn Technologies, Inc.'

# Configuration Variables
# -----------------------
secret_key = os.environ.get('PINN_SECRET_KEY', None)
"""str: The Pinn secret key value, used to authenticate API requests."""

api_host = os.environ.get('PINN_API_HOST', 'https://pinnapis.com')
"""str: Pinn API host, defaults to production Pinn-hosted service."""

api_version = os.environ.get('PINN_API_VERSION', None)
"""str: Pinn API verision string, set this to use a specific API version."""

# API
# ---
from .api.app import App  # NOQA
from .api.device import Device  # NOQA
from .api.enrollment_key import EnrollmentKey  # NOQA
from .api.event import Event  # NOQA
from .api.health import healthy  # NOQA
from .api.log import Log  # NOQA
from .api.recovery_key import RecoveryKey  # NOQA
from .api.user import User  # NOQA
from .api.webhook_endpoint import WebhookEndpoint  # NOQA
from .utils import IDToken  # NOQA

# Versioning
# ----------
from ._version import get_versions  # NOQA
__version__ = get_versions()['version']
del get_versions



"""
:copyright: (c) 2019 Pinn Technologies, Inc.
:license: MIT
"""

from ..requester import Requester
from .list import List


class Log(object):
    """Log resource and interface.

    Attributes:
        response (dict): Underlying dictionary response
        object (str): Identifier for the resource
        log_id (str): Unique ID for the log
        user_id (str): ID of the user that authenticated
        device_id (str): ID of the device that authenticated
        app_id (str): ID of the mobile app that was involved
        web_app_id (str, optional): ID of web app that was involved
        created_at (int): Unix timestamp in seconds for when the app was created
        success (bool): True if authentication was successful
        score (float): Combined score for all scorable biometrics
        factors (dict): Breakdown of all factors used an their score if applicable
        auth_type (str): Either `web` or `mobile`
    """

    OBJECT_NAME = 'log'
    endpoint = '/v1/logs'

    def __init__(self, response):
        """Initialize a user model with an API response."""
        self.response = response
        self.object = response['object']
        self.log_id = response['log_id']
        self.user_id = response['user_id']
        self.device_id = response['device_id']
        self.app_id = response['app_id']
        self.web_app_id = response['web_app_id']
        self.created_at = response['created_at']
        self.success = response['success']
        self.score = response['score']
        self.factors = response['factors']
        self.auth_type = response['auth_type']

    @classmethod
    def list(cls, limit=None, starting_after=None):
        """List created Pinn logs."""
        response = Requester.get(cls.endpoint, params={'limit': limit,
                                                       'starting_after': starting_after})
        return List(response, Log, limit)

    @classmethod
    def retrieve(cls, log_id):
        """Retrieve a log with a provided log ID."""
        return Log(Requester.get(cls.endpoint + '/' + log_id))



"""
:copyright: (c) 2019 Pinn Technologies, Inc.
:license: MIT
"""

import json
from ..requester import Requester


class Pager(object):
    """Defines an iterable List pagination protocol."""

    def __init__(self, _list):
        self.started = False
        self.starting = _list
        self.current = _list

    def __iter__(self):
        return self

    def next(self):
        if not self.started:
            self.started = True
            return self.current
        if self.current.has_more:
            self.current = List(Requester.get(self.current.url,
                                              params={'limit': self.current.limit,
                                                      'starting_after': self.current.ended_at}),
                                self.current.cls,
                                self.current.limit)
            return self.current
        raise StopIteration


class List(object):
    """Data encapsulation container for an API list response."""

    OBJECT_NAME = 'list'

    def __init__(self, response, cls, limit=10):
        """Initialize a list model with an API response."""
        self.url = response['url']
        self.count = response['count']
        self.has_more = response['has_more']
        self.object = response['object']
        self.ended_at = response['ended_at']
        self.data = [cls(item) for item in response['data']]
        self.cls = cls
        self.limit = limit
        self.pager = Pager(self)

    def next_page(self):
        return List(Requester.get(self.url,
                                  params={'limit': self.limit,
                                          'starting_after': self.ended_at}),
                    self.cls,
                    self.limit)

    def __str__(self):
        data = self.dump()
        return json.dumps(data, indent=4, sort_keys=True, separators=(',', ': '))

    def __iter__(self):
        return getattr(self, 'data', []).__iter__()

    def __len__(self):
        return getattr(self, 'data', []).__len__()

    def dump(self):
        """Dump the model to a dictionary, method matches to json.dump() behavior"""
        return {'url': self.url,
                'count': self.count,
                'has_more': self.has_more,
                'object': self.object,
                'ended_at': self.ended_at,
                'data': [item.dump() for item in self.data]}



"""
:copyright: (c) 2019 Pinn Technologies, Inc.
:license: MIT
"""

from ..requester import Requester
from ..errors import PinnError


def healthy():
    """Perform a health check against the configured Pinn host.

    Returns:
        bool: True if host is available, False otherwise
    """

    endpoint = '/health'

    try:
        Requester.get(endpoint)
        return True
    except PinnError:
        return False



"""
:copyright: (c) 2019 Pinn Technologies, Inc.
:license: MIT
"""

import json
from .list import List
from .device import Device
from .log import Log
from ..requester import Requester


class User(object):
    """User resource and interface.

    Attributes:
        response (dict): Underlying dictionary response
        object (str): Identifier for the resource
        user_id (str): Unique ID for the user
        created_at (int): Unix timestamp in seconds for when the user was created
        updated_at (int): Unix timestamp in seconds for when the user was last updated
        authenticated_at (int): Unix timestamp in seconds for when the user last authenticated
        device_enrolled (bool): True if the user has at least 1 enrolled device
        left_palm_enrolled (bool): True if the user has enrolled their left palm
        right_palm_enrolled (bool): True if the user has enrolled their right palm
        metadata (dict): Arbitrary key/value data attached to the user
    """

    OBJECT_NAME = 'user'
    endpoint = '/v1/users'

    def __init__(self, response):
        """Initialize a user model with an API response."""
        self.response = response
        self.object = response['object']
        self.user_id = response['user_id']
        self.created_at = response['created_at']
        self.updated_at = response['updated_at']
        self.authenticated_at = response['authenticated_at']
        self.device_enrolled = response['device_enrolled']
        self.left_palm_enrolled = response['left_palm_enrolled']
        self.right_palm_enrolled = response['right_palm_enrolled']
        self.metadata = response['metadata']

    def __str__(self):
        data = self.dump()
        return json.dumps(data, indent=4, sort_keys=True, separators=(',', ': '))

    def dump(self):
        """Dump the model to a dictionary, method matches to json.dump() behavior"""
        return {'user_id': self.user_id,
                'created_at': self.created_at,
                'object': self.object,
                'metadata': self.metadata}

    @classmethod
    def create(cls, metadata=None):
        """Create a new Pinn user.

        Args:
            metadata (dict, optional): The ID of the Pinn user being authorized to enroll.

        Returns:
            User: The newly created user

        Raises:
            pinn.errors.RequestFailedError: Invalid metadata dict provided
            pinn.errors.APIError: Internal server error
            pinn.errors.APIConnectionError: API is unreachable [501-503]
        """
        if metadata:
            data = {'metadata': metadata}
        else:
            data = None
        return User(Requester.post(cls.endpoint, data=data))

    @classmethod
    def list(cls, limit=None, starting_after=None):
        """List created Pinn users."""
        response = Requester.get(cls.endpoint, params={'limit': limit,
                                                       'starting_after': starting_after})
        return List(response, User, limit)

    @classmethod
    def retrieve(cls, user_id):
        """Retrieve a Pinn user.

        Args:
            user_id (str): The ID of the Pinn user to query.

        Returns:
            User: A User resource

        Raises:
            pinn.errors.RequestFailedError: User not found
            pinn.errors.APIError: Internal server error
            pinn.errors.APIConnectionError: API is unreachable [501-503]
        """
        return User(Requester.get(cls.endpoint + '/' + user_id))

    @classmethod
    def update(cls, user_id, metadata, status):
        """Update a Pinn user.

        Args:
            user_id (str): The ID of the Pinn user to update.
            metadata (dict, optional): Metadata to update
            status (str, optional): User status to update

        Returns:
            User: A User resource

        Raises:
            pinn.errors.RequestFailedError: User not found, Invalid metadata or status
            pinn.errors.APIError: Internal server error
            pinn.errors.APIConnectionError: API is unreachable [501-503]
        """
        data = {}
        if metadata:
            data['metadata'] = metadata
        if status:
            data['status'] = status
        return User(Requester.patch(cls.endpoint + '/' + user_id, data=data))

    @classmethod
    def delete(cls, user_id):
        """Update a Pinn user.

        Args:
            user_id (str): The ID of the Pinn user to delete.

        Returns:
            bool: A Deleted response

        Raises:
            pinn.errors.RequestFailedError: User not found, Invalid metadata or status
            pinn.errors.APIError: Internal server error
            pinn.errors.APIConnectionError: API is unreachable [501-503]
        """
        return Requester.delete(cls.endpoint + '/' + user_id)

    @classmethod
    def list_devices(cls, user_id, limit=None, starting_after=None):
        """List created Pinn user's devices."""
        endpoint = cls.endpoint + '/' + user_id + '/devices'
        response = Requester.get(endpoint, params={'limit': limit,
                                                   'starting_after': starting_after})
        return List(response, Device, limit)

    @classmethod
    def list_logs(cls, user_id, limit=None, starting_after=None):
        """List created Pinn users."""
        endpoint = cls.endpoint + '/' + user_id + '/logs'
        response = Requester.get(endpoint, params={'limit': limit,
                                                   'starting_after': starting_after})
        return List(response, Log, limit)



"""
:copyright: (c) 2019 Pinn Technologies, Inc.
:license: MIT
"""

import requests
from . import __title__
from ._version import get_versions
__version__ = get_versions()['version']
from .errors import PinnError, ConfigurationError


class Requester(object):
    """
    The requester class is a lightweight wrapper over python requests library.

    This class handles the request formation and response handling when interacting with
    the Pinn REST interface. It is leveraged to inject HTTP headers, handle errors and
    return JSON data.
    """

    @classmethod
    def post(cls, endpoint, data=None, params=None):
        """Performs a POST request to the given endpoint."""
        url = cls.url(endpoint)
        response = requests.post(url, json=data, params=params, headers=cls.headers())
        return cls.handle_response(response)

    @classmethod
    def get(cls, endpoint, params=None):
        """Performs a GET request to the given endpoint."""
        url = cls.url(endpoint)
        response = requests.get(url, params=params, headers=cls.headers())
        return cls.handle_response(response)

    @classmethod
    def patch(cls, endpoint, data=None, params=None):
        """Performs a PATCH request to the given endpoint."""
        url = cls.url(endpoint)
        response = requests.patch(url, json=data, params=params, headers=cls.headers())
        return cls.handle_response(response)

    @classmethod
    def delete(cls, endpoint, data=None, params=None):
        """Performs a DELETE request to the given endpoint."""
        url = cls.url(endpoint)
        response = requests.delete(url, json=data, params=params, headers=cls.headers())
        return cls.handle_response(response)

    @staticmethod
    def handle_response(response):
        """
        Handle the response of the request.

        Returns the JSON data if the request succeeded and has content. If the
        request succeeded but has no JSON content True is returned. Finally
        this method will raise a PinnError if an exception is encountered.
        """
        if response.status_code in [requests.codes.created,
                                    requests.codes.accepted,
                                    requests.codes.no_content]:
            return True
        elif response.status_code == requests.codes.ok:
            return response.json()
        else:
            raise PinnError.from_response(response)

    @staticmethod
    def headers():
        """
        Return the request headers to be sent with each request.

        Verifies before requesting that a Pinn secret key has been set.
        """
        from . import secret_key, api_version
        if secret_key is None:
            raise ConfigurationError('Pinn secret key has not been set, unable to perform requests')
        user_agent = "{}-{}".format(__title__, __version__)
        headers = {'Authorization': 'Bearer ' + secret_key,
                   'User-Agent': user_agent}
        if api_version:
            headers['Pinn-Version'] = api_version
        return headers

    @staticmethod
    def url(endpoint):
        """Return the fully qualified URL based off the api_host value and endpoint"""
        from . import api_host
        return api_host + endpoint



"""
:copyright: (c) 2019 Pinn Technologies, Inc.
:license: MIT
"""

from ..requester import Requester
from .list import List


class Event(object):
    """Event resource and interface.

    Attributes:
        response (dict): Underlying dictionary response
        object (str): Identifier for the resource
        event_id (str): Unique ID for the event
        created_at (int): Unix timestamp in seconds for when the app was created
        data (dict): Underlying data for the event
        request (str): ID of the request this event corresponds to
        event_type (str): The type of the event that occured
        livemode (bool): True if the event occured outside of sandbox environment
    """

    OBJECT_NAME = 'event'
    endpoint = '/v1/events'

    def __init__(self, response):
        """Initialize a user model with an API response."""
        self.response = response
        self.object = response['object']
        self.event_id = response['event_id']
        self.created_at = response['created_at']
        self.data = response['data']
        self.request = response['request']
        self.event_type = response['type']
        self.livemode = response['livemode']

    @classmethod
    def list(cls, limit=None, starting_after=None):
        """List created Pinn logs."""
        response = Requester.get(cls.endpoint, params={'limit': limit,
                                                       'starting_after': starting_after})
        return List(response, Event, limit)

    @classmethod
    def retrieve(cls, event_id):
        """Retrieve an event with a provided event ID."""
        return Event(Requester.get(cls.endpoint + '/' + event_id))



"""
:copyright: (c) 2019 Pinn Technologies, Inc.
:license: MIT
"""

import io
from setuptools import setup, find_packages
import versioneer

with io.open('README.md', 'r', encoding='utf-8') as f:
    readme = f.read()

setup(
    name='pinn',
    author='Pinn Technologies, Inc.',
    author_email='developers@pinn.ai',
    license="MIT",
    url="https://github.com/pinntech/pinn-python",
    zip_safe=False,
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    install_requires=[
        'requests >= 2.20; python_version >= "3.0"',
        'requests[security] >= 2.20; python_version < "3.0"',
        'pyjwt >= 1.7.1',
    ],
    tests_require=[
        "pytest >= 3.4",
        "pytest-mock >= 1.7",
        "pytest-xdist >= 1.22",
        "pytest-cov >= 2.5",
    ],
    python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*",
    project_urls={
        "Bug Tracker": "https://github.com/pinntech/pinn-python/issues",
        "Documentation": "https://pinn.readthedocs.org",
        "Source Code": "https://github.com/pinntech/pinn-python",
    },
    keywords='pinn api client library authentication',
    classifiers=[k for k in open('CLASSIFIERS').read().split('\n') if k],
    description='Python bindings for the Pinn REST API',
    long_description=readme,
    long_description_content_type='text/markdown',
    packages=find_packages(exclude=["tests*"])
)



"""
:copyright: (c) 2019 Pinn Technologies, Inc.
:license: MIT
"""

import json
from ..requester import Requester


class RecoveryKey(object):
    """Recovery key resource and interface.

    Attributes:
        response (dict): Underlying dictionary response
        object (str): Identifier for the resource
        recovery_key_id (str): Unique ID for this recovery key
        user_id (str): ID of the user who the key corresponds to
        created_at (int): Unix timestamp in seconds for when the key was created
        expires_at (int): Unix timestamp in seconds for when the key will expire
        livemode (bool): True if recovery key is in live environment
        secret (str): A secret to authenticate the recovery request from Pinn mobile SDK
        flow (list): The auth flow required for recovery
    """

    OBJECT_NAME = 'recovery_key'
    endpoint = '/v1/recovery_keys'

    def __init__(self, response):
        """Initialize a user model with an API response."""
        self.response = response
        self.object = response['object']
        self.recovery_key_id = response['recovery_key_id']
        self.user_id = response['user_id']
        self.created_at = response['created_at']
        self.expires_at = response['expires_at']
        self.livemode = response['livemode']
        self.secret = response['secret']
        self.flow = response['flow']

    def __str__(self):
        return json.dumps(self.response, indent=4, sort_keys=True, separators=(',', ': '))

    @classmethod
    def create(cls, user_id, flow):
        """Create a new recovery key for a given user.

        Args:
            user_id (str): ID of user to create the recovery key for.
            flow (list, optional): List of remote factors to check prior to re-enrollment

        Returns:
            RecoveryKey: The newly created recovery key

        Raises:
            pinn.errors.RequestFailedError: User not found, invalid `flow` provided
            pinn.errors.APIError: Internal server error
            pinn.errors.APIConnectionError: API is unreachable [501-503]
        """
        data = {'user_id': user_id, 'flow': flow}
        return RecoveryKey(Requester.post(cls.endpoint, data=data))



"""
:copyright: (c) 2019 Pinn Technologies, Inc.
:license: MIT
"""

import json
from ..requester import Requester
from .list import List


class App(object):
    """App resource and interface.

    Attributes:
        response (dict): Underlying dictionary response
        object (str): Identifier for the resource
        app_id (str): Unique ID for the app
        created_at (int): Unix timestamp in seconds for when the app was created
        updated_at (int): Unix timestamp in seconds for when the app was last updated
        publishable_key (str): Non-secret key that is embedded to configure the SDK
        app_type (str): Either `ios`, `android` or `web`
        name (str): Name of the app
        description (str): Optional app description text
    """

    OBJECT_NAME = 'app'
    endpoint = '/v1/apps'

    def __init__(self, response):
        """Initialize a app model with an API response."""
        self.response = response
        self.object = response['object']
        self.app_id = response['app_id']
        self.created_at = response['created_at']
        self.updated_at = response['updated_at']
        self.publishable_key = response['publishable_key']
        self.app_type = response['app_type']
        self.name = response['name']
        self.description = response['description']

    def __str__(self):
        data = self.dump()
        return json.dumps(data, indent=4, sort_keys=True, separators=(',', ': '))

    def dump(self):
        """Dump the model to a dictionary, method matches to json.dump() behavior"""
        return {'app_id': self.app_id,
                'created_at': self.created_at,
                'object': self.object,
                'publishable_key': self.publishable_key,
                'app_type': self.app_type,
                'name': self.name,
                'description': self.description}

    @classmethod
    def create(cls, app_type, name, description=None):
        """Create a new Pinn iOS/Android/Web app.

        Args:
            app_type (str): Either `ios`, `android` or `web`
            name (str): Human readable name for the app
            description (str, optional): Descriptive text for the app

        Returns:
            App: Newly created app resource

        Raises:
            pinn.errors.APIError: Internal server error
            pinn.errors.APIConnectionError: API is unreachable [501-503]
        """
        assert app_type in ("android", "ios", "web"), "App must be android, ios, or web"
        data = {'name': name,
                'description': description}
        return App(Requester.post(cls.endpoint + '/' + app_type, data=data))

    @classmethod
    def list(cls, limit=None, starting_after=None):
        """List created Pinn apps."""
        response = Requester.get(cls.endpoint, params={'limit': limit,
                                                       'starting_after': starting_after})
        return List(response, App, limit)

    @classmethod
    def retrieve(cls, app_id):
        """Retrieve a Pinn iOS/Android/Web app.

        Args:
            app_id (str): ID of the app to query.

        Returns:
            App: The queried app resource

        Raises:
            pinn.errors.RequestFailedError: App not found
            pinn.errors.APIError: Internal server error
            pinn.errors.APIConnectionError: API is unreachable [501-503]
        """
        return App(Requester.get(cls.endpoint + '/' + app_id))

    @classmethod
    def delete(cls, app_id):
        """Delete a Pinn iOS/Android/Web app.

        Note:
            This may break your integration if app is currently in use live.

        Args:
            app_id (str): ID of the app to delete.

        Returns:
            Deleted: A deleted response

        Raises:
            pinn.errors.RequestFailedError: App not found
            pinn.errors.APIError: Internal server error
            pinn.errors.APIConnectionError: API is unreachable [501-503]
        """
        return Requester.delete(cls.endpoint + '/' + app_id)
